import java.util.*;

public class MySpielfeld implements Spielfeld {

	private Stein[][] feld = null;

	public MySpielfeld(int spalten, int zeilen) {
		feld = new Stein[spalten][zeilen];
	}

	/** Anzeigen des Spielfeldes.
		*/
	public void anzeigen() {
	//
	// |   |   |   |   |   |   |   |   |   |   |   |   |
	// | X |   |   |   |   |   |   |   |   |   |   |   |
	// |---|---|---|---|---|---|---|---|---|---|---|---|
	// | O |   |   |   |   |   |   |   |   |   |   |   |
	// |---|---|---|---|---|---|---|---|---|---|---|---|
	// | X |   |   |   | X |   |   |   |   |   |   |   |
	// |---|---|---|---|---|---|---|---|---|---|---|---|
	// | O |   |   | O | O | O |   |   |   |   |   |   |
	// |---|---|---|---|---|---|---|---|---|---|---|---|
	// | X |   |   | X | X | X |   |   |   |   |   |   |
	// |---|---|---|---|---|---|---|---|---|---|---|---|
	// | O |   | X | X | X | O |   |   |   |   |   |   |
	// |---|---|---|---|---|---|---|---|---|---|---|---|
	// | X | X | O | X | X | O |   |   |   |   |   |   |
	// =================================================
	// | 0 | 1 | 2 | 3 | 4 | 5 | 6 | 7 | 8 | 9 | 10| 11|

		for (int s=0;s<spalten();s++)
			System.out.print("|   ");
		System.out.print("|\n");

		for (int z=zeilen()-1;z>=0;z--) {
			for (int s=0;s<spalten();s++) {
				Stein stein = feld[s][z];
				if (stein == null) System.out.print("|   ");
				else {
					System.out.print("| "+((stein.getFarbe() == Stein.ROT)?"X":"O")+" ");
				}
			}
			System.out.print("|\n");
			for (int s=0;s<spalten();s++)
				System.out.print("|---");
			System.out.print("|\n");
		}

		for (int s=0;s<spalten();s++)
			System.out.print("====");
		System.out.print("=\n");
		for (int s=0;s<spalten();s++)
			System.out.print("| "+((s<10)?(s+" "):(s+"")));
		System.out.print("|\n");
	}


	/** Setzen eines Steines in eine Spalte.
		* @param spalte Die Spalte, in der ein Stein gesetzt werden soll.
		* @param stein Der Stein, der gesetzt werden soll.
		* @return Liefert die Zeile des gesetzten Stein zurck.
		* @throws SpalteVollException wenn der Stein nicht mehr in die
		*         Spalte pat.
		*/
	public int setzeStein(int spalte, Stein stein) {
		int rv = -1;
		for (int z = 0; z < zeilen(); z++) {
			if (feld[spalte][z] == null) {
				rv = z;
				feld[spalte][z] = stein;
				return rv;
			}
		}
		throw new SpalteVollException();
	}


	/** Prfen ob eine Position eine Gewinn-Position ist.
		* @param spalte Die Spalten-Position zum berprfen.
		* @param zeile Die Zeilen-Position zum berprfen.
		* @return Liefert <tt>true</tt> wenn an einer Gewinn-Position,
		*         ansonsten <tt>false</tt>.
		* @throws IllegalArgumentException wenn an der Position
		*         kein Stein vorhanden ist.
		*/
	public boolean pruefeStein(int spalte, int zeile) {
		Stein stein = feld[spalte][zeile];
		if (stein == null)
			throw new IllegalArgumentException(
				"An der Position ("+spalte+"/"+zeile+") ist kein Stein");

		int farbe = stein.getFarbe();

		// temp Zeilenvariable
		int z;
		// temp Spaltenvariable
		int s;

		// -------------------------------------------------------------
		// berprfen von horizontal -----------------------------------
		int counter = 1;
		for (int i=1;i<4;i++) {
			s = spalte - i;
			// wenn am linken Spielrand, dann breche Schleife ab
			if (s<0) break;

			// wenn an (s/zeile) ein Stein mit gleicher Farbe,
			// dann erhhe Counter
			// ansonsten breche Schleife ab
			if (feld[s][zeile] != null && feld[s][zeile].getFarbe() == farbe) {
				counter++;
			} else break;
		}
		for (int i=1;i<4;i++) {
			s = spalte + i;
			// wenn am rechten Spielrand, dann breche Schleife ab
			if (s>=spalten()) break;

			// wenn an (s/zeile) ein Stein mit gleicher Farbe,
			// dann erhhe Counter
			// ansonsten breche Schleife ab
			if (feld[s][zeile] != null && feld[s][zeile].getFarbe() == farbe) {
				counter++;
			} else break;
		}
		// wenn counter grer gleich 4, dann liefere true zurck.
		if (counter >= 4) return true;

		// -------------------------------------------------------------
		// berprfen von vertikal   -----------------------------------
		counter = 1;
		for (int i=1;i<4;i++) {
			z = zeile - i;
			// wenn am unteren Spielrand, dann breche Schleife ab
			if (z<0) break;

			// wenn an (spalte / z) ein Stein mit gleicher Farbe,
			// dann erhhe Counter
			// ansonsten breche Schleife ab
			if (feld[spalte][z] != null && feld[spalte][z].getFarbe() == farbe) {
				counter++;
			} else break;
		}
		for (int i=1;i<4;i++) {
			z = zeile + i;
			// wenn am oberen Spielrand, dann breche Schleife ab
			if (z>=zeilen()) break;

			// wenn an (spalte / z) ein Stein mit gleicher Farbe,
			// dann erhhe Counter
			// ansonsten breche Schleife ab
			if (feld[spalte][z] != null && feld[spalte][z].getFarbe() == farbe) {
				counter++;
			} else break;
		}
		// wenn counter grer gleich 4, dann liefere true zurck.
		if (counter >= 4) return true;

		// -------------------------------------------------------------
		// berprfen von diagonal linksoben nach rechtunten -----------
		counter = 1;
		for (int i=1;i<4;i++) {
			z = zeile + i;
			s = spalte -i;
			// wenn am oberen oder am linken Spielrand,
			// dann breche Schleife ab
			if (z>=zeilen() || s <0) break;

			// wenn an (s / z) ein Stein mit gleicher Farbe,
			// dann erhhe Counter
			// ansonsten breche Schleife ab
			if (feld[s][z] != null && feld[s][z].getFarbe() == farbe) {
				counter++;
			} else break;
		}
		for (int i=1;i<4;i++) {
			z = zeile - i;
			s = spalte +i;
			// wenn am unteren oder am rechten Spielrand,
			// dann breche Schleife ab
			if (z<0 || s >=spalten()) break;

			// wenn an (s / z) ein Stein mit gleicher Farbe,
			// dann erhhe Counter
			// ansonsten breche Schleife ab
			if (feld[s][z] != null && feld[s][z].getFarbe() == farbe) {
				counter++;
			} else break;
		}
		// wenn counter grer gleich 4, dann liefere true zurck.
		if (counter >= 4) return true;

		// -------------------------------------------------------------
		// berprfen von diagonal linksunten nach rechtoben -----------
		counter = 1;
		for (int i=1;i<4;i++) {
			z = zeile - i;
			s = spalte -i;
			// wenn am unteren oder am linken Spielrand,
			// dann breche Schleife ab
			if (z<0 || s<0) break;


			// wenn an (s / z) ein Stein mit gleicher Farbe,
			// dann erhhe Counter
			// ansonsten breche Schleife ab
			if (feld[s][z] != null && feld[s][z].getFarbe() == farbe) {
				counter++;
			} else break;
		}
		for (int i=1;i<4;i++) {
			z = zeile + i;
			s = spalte +1;
			// wenn am oberen oder am rechten Spielrand,
			// dann breche Schleife ab
			if (z>=zeilen() || s >=spalten()) break;

			// wenn an (s / z) ein Stein mit gleicher Farbe,
			// dann erhhe Counter
			// ansonsten breche Schleife ab
			if (feld[s][z] != null && feld[s][z].getFarbe() == farbe) {
				counter++;
			} else break;
		}
		// wenn counter grer gleich 4, dann liefere true zurck.
		if (counter >= 4) return true;

		// Counter war nie grer gleich 4
		return false;
	}


	/** Entfernt vom Spielfeld alle Steine.
		* @return Liefert eine Collection mit den Steinen zurck.
		*/
	public Collection leeren() {
		Vector freieSteine = new Vector();
		for (int s = 0; s<spalten(); s++) {
			for (int z = 0; z<zeilen(); z++) {
				if (feld[s][z] != null) freieSteine.add(feld[s][z]);
			}
		}
		return freieSteine;
	}


	/** Liefert die Anzahl der Spalten vom Spielfeld zurck.
		* @return Die Anzahl der Spalten.
		*/
	public int spalten() {
		return feld.length;
	}


	/** Liefert die Anzahl der Zeilen vom Spielfeld zurck.
		* @return Die Anzahl der Zeilen.
		*/
	public int zeilen() {
		return feld[0].length;
	}


	/** berpft, ob das Spielfeld vollstndig gefllt ist.
		* @return Liefert <tt>true</tt>, wenn alle Zeilen und
		*         Spalten vollstndig aufgefllt sind.
		*/
	public boolean voll() {
		boolean rv = true;
		for (int s = 0; s<spalten();s++) {
			if (feld[s][zeilen()-1] == null) {
				rv = false;
				break;
			}
		}
		return rv;
	}

}