import java.util.*;

public class MyTopf implements Topf {

	// Der eigentliche Topf ;o)
	Vector steine = new Vector();

	/** Erzeugt einen neuen Top mit der angegebenen Anzahl
		* von Steinen fr 2 Spieler.
		* Steine haben die Farbe Stein.ROT oder Steine.BLAU.
		* @param anzahlProSpieler Die Anzahl der Steine pro Spieler
		* @throws IllegalArgumentException wenn die Anzahl der Steine
		*         kleiner als 4 ist.
		*/
	public MyTopf(int anzahlProSpieler) {
		if (anzahlProSpieler < 4)
			throw new IllegalArgumentException(
				"Mindestens 4 Steine pro Spieler. Angegeben wurden "+anzahlProSpieler+".");

		for (int i=0; i<anzahlProSpieler; i++) {
			Stein rotStein  = new MyStein(Stein.ROT);
			Stein blauStein = new MyStein(Stein.BLAU);
			steine.add(rotStein);
			steine.add(blauStein);
		}
	}


	/** Liefert einen Stein der gegebenen Farbe aus dem Topf zurck.
		* @param farbe Die Farbe des Steins.
		* @return Liefert einen Stein der gegebenen Farbe zurck.
		* @throws IllegalArgumentException wenn die Farbe nicht
		*         Stein.ROT oder Stein.BLAU ist.
		* @throws TopfLeerException wenn kein Stein mehr im Topf vorhanden ist.
		* @throws KeinSteinException wenn kein Stein der angebenen Farbe
		*         mehr vorhanden ist.
		*/
	public Stein getStein(int farbe) {
		if (farbe != Stein.ROT && farbe != Stein.BLAU)
			throw new IllegalArgumentException("Ungltige Farbe "+farbe+".");

		Stein rv = null;

		Stein st = null;

		if (steine.isEmpty()) throw new TopfLeerException();

		for (int i=0; i< steine.size(); i++) {
			if (steine.get(i) instanceof Stein) {
				st = (Stein) steine.get(i);
				if (st.getFarbe() == farbe) {
					rv = (Stein) steine.remove(i);
					break;
				}
			}
		}

		if (rv == null) throw new KeinSteinException();

		return rv;
	}


	/** Packt ein Stein zurck in den Topf.
		* @param stein Der Stein, der zurck in den Topf kommt.
		* @throws IllegalArgumentException wenn der Stein null ist.
		*/
	public void setStein(Stein stein) {
		if (stein == null)
			throw new IllegalArgumentException("Es darf nicht null bergeben werden.");
		steine.add(stein);
	}


	/** Packt Steine zurck in den Topf.
		* @param steine Die Steine, die zurck in den Topf kommen.
		* @throws IllegalArgumentException wenn die Collection null ist.
		*/
	public void setStein(Collection steinCol) {
		if (steinCol == null)
			throw new IllegalArgumentException("Es darf nicht null bergeben werden.");
		steine.addAll(steinCol);
	}


	/** Liefert die Menge der vorhandenen Steine einer Farbe aus
		* dem Topf zurck.
		* @param farbe Die Farbe der Steine, dessen Anzahl gesucht wird.
		* @return Die Anzahl der Steine fr die Farbe.
		* @throws IllegalArgumentException wenn die Farbe nicht
		*         Stein.ROT oder Stein.BLAU ist.
		*/
	public int anzahlSteine(int farbe) {
		if (farbe != Stein.ROT && farbe != Stein.BLAU)
			throw new IllegalArgumentException("Ungltige Farbe "+farbe+".");

		int rv = 0;
		Stein st = null;

		for (int i=0; i<steine.size(); i++) {
			if (steine.get(i) instanceof Stein) {
				st = (Stein) steine.get(i);
				if (st.getFarbe() == farbe) rv++;
			}
		}
		return rv;
	}
}